# ExecWrapper.pm:
#
# Run an external program, with the supplied commandline,
# using filtered I/O and timeouts. Optionally, run a
# series of callbacks on the each line.
#
# Copyright (c) 2005, Taco Hoekwater

package ExecWrapper;

use IPC::Open2;
use IO::Select;
use IO::Handle qw(_IOLBF);
use Time::HiRes qw(gettimeofday tv_interval);

use POSIX ":sys_wait_h";

sub new {
  my ($class,$timeout) = @_;
  $timeout ||= 60;
  my $this = {'timeout' => $timeout};
  bless $this, $class;
  return $this;
}


sub start {
  my ($this,@cmdline) = @_;
  # the file selectors are auto-instantiated
  $pid = open2(\*STDINI,\*STDOUTI,@cmdline) || return 0;
  #
  $stdin = IO::Handle->new_from_fd(fileno(\*STDINI),"r");
  $stdin->blocking(0);
  $stdout = IO::Handle->new_from_fd(fileno(\*STDOUTI),"w") ;
  $stdout->autoflush(1);
  #
  $this->{'pid'}    = $pid;
  $this->{'stdin'}  = $stdin;
  $this->{'stdout'} = $stdout;
  # return success if ok:
  return $pid;
}


sub run {
  my $this = shift;
  my $line;
  my $tstart = [gettimeofday];
  while (waitpid ($this->{pid}, WNOHANG) < 1) {
	$line = $this->{stdin}->getline();
	if (defined($line)) {
	  $this->do_line($line);
	  $tstart = [gettimeofday];
	} else {
	  if (tv_interval($tstart) > $this->{timeout}) {
		# kill process
		my $pid = $this->{pid};
		print STDERR "Killing process $pid ...\n";
		kill 9, $pid;
		return;
	  }
	}
  }
  while ($line = $this->{stdin}->getline()) {
   $this->do_line($line);
  }
  $this->{stdin}->close();
  $this->{stdout}->close();
}


sub do_line {
  my ($this,$line) = @_;
  foreach my $linetype (@{$this->{types}}) {
	my ($match,$code) = @$linetype;
    if ($line =~ /$match/) {
	  &{$code} ($this,$line);
	  last;
	}
  }
}

sub types {
  my $this = shift;
  $this->{types} = [@_];
}

1;
